using System;

namespace TX_Text_Control_Words
{
	public class FileHandler
	{
		//
		// File handling in TX Text Control Words
		//
		// The sample program works in 2 modes, Default and XML mode. XML mode is entered when 
		// an  XML file is loaded. An XML file can only be saved as XML, but can be exported to 
		// all other formats.
		//
		// These file types will be available in the Save and Export dialog boxes. Save types are
		// are read/write, whereas Export types are those which cannot be loaded back in. When
		// a file is exported, the original file name and type are preserved.
		//
		const TXTextControl.StreamType DefaultExportTypes = TXTextControl.StreamType.AdobePDF | TXTextControl.StreamType.CascadingStylesheet;
		const TXTextControl.StreamType DefaultSaveTypes   = TXTextControl.StreamType.All & ~DefaultExportTypes & ~TXTextControl.StreamType.XMLFormat;
		const TXTextControl.StreamType DefaultInsertTypes = TXTextControl.StreamType.All & ~DefaultExportTypes & ~TXTextControl.StreamType.XMLFormat;
		const TXTextControl.StreamType XMLModeSaveTypes   = TXTextControl.StreamType.XMLFormat;
		const TXTextControl.StreamType XMLModeExportTypes = TXTextControl.StreamType.All & ~XMLModeSaveTypes;
		const TXTextControl.StreamType XMLModeInsertTypes = TXTextControl.StreamType.PlainAnsiText | TXTextControl.StreamType.PlainText;

		private string m_DocumentFileName = "";
		private TXTextControl.StreamType m_DocumentStreamType;
		private TXTextControl.TextControl m_TextControl;
		private bool m_DocumentDirty;

		// CSS related options
		private string m_CSSFileName = "";
		private TXTextControl.CssSaveMode m_CSSSaveMode;

		// PDF related options
		private string m_PDFMasterPassword = "";
		private string m_PDFUserPassword = "";
		private TXTextControl.DocumentAccessPermissions m_PDFDocumentAccessPermissions = 
			TXTextControl.DocumentAccessPermissions.AllowAll;

		public TXTextControl.TextControl TextControl
		{
			set { m_TextControl = value; }
		}

		public string DocumentFileName
		{
			set { m_DocumentFileName = value; }
			get {return m_DocumentFileName; }
		}

		public bool InXMLMode
		{
			get { return (m_DocumentStreamType == TXTextControl.StreamType.XMLFormat); }
		}

		public bool DocumentDirty
		{
			get { return  m_DocumentDirty; }
			set { m_DocumentDirty = value; }
		}

		public string CSSFileName
		{
			get { return m_CSSFileName; }
			set { m_CSSFileName = value; }
		}

		public TXTextControl.CssSaveMode CSSSaveMode
		{
			get { return m_CSSSaveMode; }
			set { m_CSSSaveMode = value; }
		}

		public string PDFMasterPassword
		{
			set { m_PDFMasterPassword = value; }
			get {return m_PDFMasterPassword; }
		}
		public string PDFUserPassword
		{
			set { m_PDFUserPassword = value; }
			get {return m_PDFUserPassword; }
		}
		public TXTextControl.DocumentAccessPermissions PDFDocumentAccessPermissions
		{
			set { m_PDFDocumentAccessPermissions = value; }
			get {return m_PDFDocumentAccessPermissions; }
		}

		public bool FileOpen()
		{
			bool Succeeded = false;

			TXTextControl.LoadSettings LoadSettings = new TXTextControl.LoadSettings();

			if(m_DocumentFileName == "")
				m_TextControl.Load(TXTextControl.StreamType.All, LoadSettings);
			else
			{
				if(m_DocumentFileName.ToLower().EndsWith(".rtf"))
					m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.RichTextFormat, LoadSettings);
				else if(m_DocumentFileName.ToLower().EndsWith(".doc"))
					m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.MSWord, LoadSettings);
				else if(m_DocumentFileName.ToLower().EndsWith(".txt"))
					m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.PlainText, LoadSettings);
				else if(m_DocumentFileName.ToLower().EndsWith(".htm") || m_DocumentFileName.EndsWith(".html"))
					m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.HTMLFormat, LoadSettings);
			}
	
			if (LoadSettings.LoadedFile != "")
			{
				m_DocumentFileName = LoadSettings.LoadedFile;
				m_DocumentStreamType = LoadSettings.LoadedStreamType;
				m_DocumentDirty = false;
				m_TextControl.PageMargins = LoadSettings.PageMargins;
				m_TextControl.PageSize = LoadSettings.PageSize;
				m_CSSFileName = LoadSettings.CssFileName;
				m_CSSSaveMode = TXTextControl.CssSaveMode.None;

				Succeeded = true;
			}

			return Succeeded;
		}

		public void FileSave()
		{
			TXTextControl.SaveSettings SaveSettings = new TXTextControl.SaveSettings();

			SaveSettings.PageMargins = m_TextControl.PageMargins;
			SaveSettings.PageSize = m_TextControl.PageSize;
			SaveSettings.CssFileName = m_CSSFileName;
			SaveSettings.CssSaveMode = m_CSSSaveMode;

			if (DocumentFileName != null &&  DocumentFileName != "")
			{
				// save under same name and type
				m_TextControl.Save(DocumentFileName, m_DocumentStreamType, SaveSettings);
			}
			else
			{
				// save as..
				if(InXMLMode)
					m_TextControl.Save(XMLModeSaveTypes, SaveSettings);
				else
					m_TextControl.Save(DefaultSaveTypes, SaveSettings);
			}

			if(SaveSettings.SavedFile != "")
			{
				m_DocumentFileName = SaveSettings.SavedFile;
				m_DocumentStreamType = SaveSettings.SavedStreamType;
				m_DocumentDirty = false;
			}
		}

		public void FileSaveAs()
		{
			TXTextControl.SaveSettings SaveSettings = new TXTextControl.SaveSettings();

			SaveSettings.PageMargins = m_TextControl.PageMargins;
			SaveSettings.PageSize = m_TextControl.PageSize;
			SaveSettings.CssFileName = m_CSSFileName;
			SaveSettings.CssSaveMode = m_CSSSaveMode;

			if (InXMLMode)
				m_TextControl.Save(XMLModeSaveTypes, SaveSettings);
			else
				m_TextControl.Save(DefaultSaveTypes, SaveSettings);

			if (SaveSettings.SavedFile != "")
			{
				if (m_DocumentStreamType != SaveSettings.SavedStreamType)
				{
					// reload file if saved in a different format
					m_TextControl.ResetContents();
					m_TextControl.Load(SaveSettings.SavedFile, SaveSettings.SavedStreamType);
				}
				m_DocumentFileName = SaveSettings.SavedFile;
				m_DocumentStreamType = SaveSettings.SavedStreamType;
				m_DocumentDirty = false;
			}
		}

		public void Export()
		{
			TXTextControl.SaveSettings SaveSettings = new TXTextControl.SaveSettings();

			SaveSettings.UserPassword = m_PDFUserPassword;
			SaveSettings.MasterPassword = m_PDFMasterPassword;
			SaveSettings.DocumentAccessPermissions = m_PDFDocumentAccessPermissions;

			if (InXMLMode)
				m_TextControl.Save(XMLModeExportTypes,SaveSettings);
			else
				m_TextControl.Save(DefaultExportTypes, SaveSettings);
		}

		public void Insert()
		{
			TXTextControl.LoadSettings LoadSettings = new TXTextControl.LoadSettings();

			if (InXMLMode)
				m_TextControl.Selection.Load(XMLModeInsertTypes, LoadSettings);
			else
				m_TextControl.Selection.Load(DefaultInsertTypes, LoadSettings);

			if (LoadSettings != null && LoadSettings.LoadedFile != "")
				m_DocumentDirty = true;
		}
	}
}
